from assemblyline import odm


@odm.model(description="Antivirus Ontology Model")
class Antivirus(odm.Model):
    @odm.model(description="Antivirus Detection Model")
    class Detection(odm.Model):
        engine_name = odm.Keyword(description="Name of antivirus engine")
        engine_version = odm.Optional(odm.Keyword(), description="Version of antivirus engine")
        engine_definition_version = odm.Optional(odm.Keyword(), description="Version of definition set")
        virus_name = odm.Optional(odm.Keyword(), description="The name of the virus")
        # What category does the verdict fall under?
        category = odm.Optional(odm.Enum(['type-unsupported',
                                          'undetected',
                                          'failure',
                                          'suspicious',
                                          'malicious']),
                                description="What category does the verdict fall under?<br><ul>"
                                "<li>`type-unsupported`: File sent to antivirus is unsupported</li>"
                                "<li>`undetected`: File not detected by antivirus</li>"
                                "<li>`failure`: Antivirus failed during detection</li>"
                                "<li>`suspicious`: Antivirus deems suspicious</li>"
                                "<li>`malicious`: Antivirus deems malicious</li></ul>")

    odm_version = odm.Text(default="2.0", description="Version of antivirus ontological result")
    detections = odm.List(odm.Compound(Detection), description="List of antivirus detections")



from assemblyline import odm


@odm.model(description="Details about the characteristics used to identify an object")
class ObjectID(odm.Model):
    guid = odm.Text(description="The GUID associated with the object")
    tag = odm.Optional(odm.Text(), description="The normalized tag of the object")
    treeid = odm.Optional(odm.Text(), description="The hash of the tree ID")
    processtree = odm.Optional(odm.Keyword(), description="Human-readable tree ID (concatenation of tags)")
    time_observed = odm.Optional(odm.Date(), description="The time at which the object was observed")



from assemblyline import odm
from assemblyline.common import forge
from assemblyline.odm.models.tagging import Tagging
from assemblyline.odm.models.ontology.types import Antivirus, PE, Sandbox

Classification = forge.get_classification()


# Result Metadata
@odm.model(index=False, store=False, description="Service Result Ontology Header (filled in by Assemblyline)")
class ResultOntologyHeader(odm.Model):

    @odm.model(index=False, store=False, description="Details about the Heuristics raised by a service")
    class HeuristicDetails(odm.Model):
        name = odm.Text(description="Name of the heuristic raised.")
        tags = odm.Compound(Tagging, description="Tags associated to heuristic")

    # Required metadata
    md5 = odm.MD5(description="MD5 of file")
    sha1 = odm.SHA1(description="SHA1 of file")
    sha256 = odm.SHA256(description="SHA256 of file")
    type = odm.Keyword(description="Type of file as identified by Assemblyline")
    size = odm.Integer(description="Size of the file in bytes")
    classification = odm.Keyword(default=Classification.UNRESTRICTED,
                                 description="Classification of the service result")
    service_name = odm.Keyword(description="Service Name")
    service_version = odm.Keyword(description="Service Version")
    service_tool_version = odm.Optional(odm.Keyword(default=''), description="Service Tool Version")

    # Optional metadata
    filenames = odm.Optional(odm.List(odm.Text()), description="Known filenames associated to file")
    date = odm.Optional(odm.Date(), description="Date of analysis")
    parent = odm.Optional(odm.SHA256(), description="Immediate parent of file relative to submission")
    sid = odm.Optional(odm.Keyword(), description="Submission ID associated to file")
    source_system = odm.Optional(odm.Text(), description="Which Assemblyline instance does the result originate from?")
    original_source = odm.Optional(odm.Text(), description="Source as specified by submitter (from metadata)")
    submitted_classification = odm.Keyword(default=Classification.UNRESTRICTED, description="Submitted classification")
    submitter = odm.Optional(odm.Keyword(), description="Submitter")
    retention_id = odm.Optional(odm.Keyword(), description="Reference to knowledge base for long-term data retention.")
    # What tags did the service associate to the result
    tags = odm.Optional(odm.Compound(Tagging), description="Tags raised by service")
    # What tags are related to certain heuristics raised
    # {
    #   "SERVICENAME_1": {
    #       "name": "Bad Things happened"
    #       "tags": {
    #           "network": {
    #               "static": {
    #                   "uri": ["bad.domain", ...]
    #                   ...
    #               }
    #               ...
    #           }
    #           ...
    #       }
    #   }
    # }
    heuristics = odm.Optional(odm.Mapping(odm.Compound(HeuristicDetails)), description="Heuristics raised by service.")


# ResultOntology
@odm.model(index=False, store=False, description="Service Result Ontology")
class ResultOntology(odm.Model):
    header = odm.Compound(ResultOntologyHeader, description="Result Ontology Header")
    antivirus = odm.Optional(odm.List(odm.Compound(Antivirus)), description="List of Antivirus Ontologies")
    pe = odm.Optional(odm.List(odm.Compound(PE)), description="List of PE Ontologies")
    sandbox = odm.Optional(odm.List(odm.Compound(Sandbox)), description="List of Sandbox Ontologies")



from assemblyline import odm
from assemblyline.odm.models.result import Attack
from assemblyline.odm.models.ontology.types.process import Process
from assemblyline.odm.models.ontology.types.network import NetworkConnection, NetworkDNS, NetworkHTTP


@odm.model(description="Sandbox Ontology Model")
class Sandbox(odm.Model):
    @odm.model(description="The metadata of the analysis, per analysis")
    class AnalysisMetadata(odm.Model):
        @odm.model(description="The metadata regarding the machine where the analysis took place")
        class MachineMetadata(odm.Model):
            ip = odm.Optional(odm.IP(), description="The IP of the machine used for analysis")
            hypervisor = odm.Optional(odm.Keyword(), description="The hypervisor of the machine used for analysis")
            hostname = odm.Optional(odm.Keyword(), description="The name of the machine used for analysis")
            platform = odm.Optional(odm.Platform(), description="The platform of the machine used for analysis")
            version = odm.Optional(odm.Keyword(),
                                   description="The version of the operating system of the machine used for analysis")
            architecture = odm.Optional(odm.Processor(),
                                        description="The architecture of the machine used for analysis")

        task_id = odm.Optional(odm.Keyword(), description="The ID used for identifying the analysis task")
        start_time = odm.Date(description="The start time of the analysis")
        end_time = odm.Date(description="The end time of the analysis")
        routing = odm.Optional(odm.Keyword(),
                               description="The routing used in the sandbox setup (Spoofed, Internet, Tor, VPN)")
        machine_metadata = odm.Optional(odm.Compound(MachineMetadata), description="The metadata of the analysis")

    @odm.model(description="A signature that was raised during the analysis of the task")
    class Signature(odm.Model):

        @odm.model(description="The subject of the signature, aka something interesting that the signature was raised on that is worth reporting")
        class Subject(odm.Model):
            ip = odm.Optional(odm.IP(), description="Subject's IP")
            domain = odm.Optional(odm.Domain(), description="Subject's domain")
            uri = odm.Optional(odm.URI(), description="Subject's URI")
            process = odm.Optional(odm.Compound(Process), description="Subject's process")
            file = odm.Optional(odm.Text(), description="Subject's file")
            registry = odm.Optional(odm.Text(), description="Subject's registry key")

        name = odm.Keyword(description="The name of the signature")
        process = odm.Optional(odm.Compound(Process), description="The process associated with the signature")
        subjects = odm.Optional(odm.List(odm.Compound(Subject)),
                                description="A list of subjects. A signature can have more than one subject.")
        description = odm.Optional(odm.Keyword(), description="The description of the signature")
        attack = odm.Optional(odm.List(odm.Compound(Attack)),
                              description="A list of Att&ck patterns and categories of the signature")

    analysis_metadata = odm.Compound(AnalysisMetadata, description="Metadata for the analysis")
    processes = odm.List(odm.Compound(Process), default=[], description="A list of processes")
    network_connections = odm.List(odm.Compound(NetworkConnection), default=[],
                                   description="The IP traffic observed during analysis")
    signatures = odm.List(odm.Compound(Signature), default=[], description="Signatures that the file may have")
    network_dns = odm.List(odm.Compound(NetworkDNS), default=[], description="The DNS traffic observed during analysis")
    network_http = odm.List(odm.Compound(NetworkHTTP), default=[],
                            description="The HTTP traffic observed during analysis")
    sandbox_name = odm.Keyword(description="The name of the sandbox")
    sandbox_version = odm.Optional(odm.Keyword(), description="The version of the sandbox")
    odm_version = odm.Text(default="1.0", description="Version of sandbox ontological result")



from assemblyline import odm
from assemblyline.odm.models.ontology.types.process import Process
from assemblyline.odm.models.ontology.types.objectid import ObjectID


@odm.model(description="Details for a low-level network connection by IP")
class NetworkConnection(odm.Model):
    objectid = odm.Compound(ObjectID, description="The object ID of the process object")
    process = odm.Optional(odm.Compound(Process), description="The process that spawned the network connection")
    source_ip = odm.Optional(odm.IP(), description="The source IP of the connection")
    source_port = odm.Optional(odm.Integer(), description="The source port of the connection")
    destination_ip = odm.IP(description="The destination IP of the connection")
    destination_port = odm.Integer(description="The destination port of the connection")
    transport_layer_protocol = odm.Enum(["tcp", "udp"], description="The transport layer protocol of the connection")
    direction = odm.Enum(["outbound", "inbound", "unknown"], description="The direction of the network connection")


@odm.model(description="Details for a DNS request")
class NetworkDNS(odm.Model):
    connection_details = odm.Compound(NetworkConnection, description="The low-level details of the DNS request")
    domain = odm.Domain(description="The domain requested")
    resolved_ips = odm.List(odm.IP(), description="A list of IPs that were resolved")
    lookup_type = odm.Text(description="The type of DNS request")


@odm.model(description="Details for an HTTP request")
class NetworkHTTP(odm.Model):
    connection_details = odm.Compound(NetworkConnection, description="The low-level details of the HTTP request")
    request_uri = odm.URI(description="The URI requested")
    request_headers = odm.Mapping(odm.Json(), description="Headers included in the request")
    request_body = odm.Optional(odm.Text(), description="The body of the request")
    request_method = odm.Enum([
        # Standard HTTP methods
        "GET", "POST", "PUT", "DELETE", "HEAD", "CONNECT", "OPTIONS", "TRACE", "PATCH",
        # WebDAV HTTP methods
        "BCOPY", "BDELETE", "BMOVE", "BPROPFIND", "BPROPPATCH", "COPY", "DELETE", "LOCK", "MKCOL", "MOVE",
        "NOTIFY", "POLL", "PROPFIND", "PROPPATCH", "SEARCH", "SUBSCRIBE", "UNLOCK", "UNSUBSCRIBE", "X-MS-ENUMATTS"
    ], description="The method of the request")
    response_headers = odm.Mapping(odm.Json(), description="Headers included in the response")
    response_status_code = odm.Optional(odm.Integer(), description="The status code of the response")
    response_body = odm.Optional(odm.Text(), description="The body of the response")



from assemblyline import odm
from assemblyline.odm.models.ontology.types.objectid import ObjectID


@odm.model(description="Details about a process")
class Process(odm.Model):
    objectid = odm.Compound(ObjectID, description="The object ID of the process object")

    # Parent process details
    pobjectid = odm.Compound(ObjectID, description="The object ID of the parent process object")
    pimage = odm.Optional(odm.Text(), description="The image of the parent process that spawned this process")
    pcommand_line = odm.Optional(odm.Text(), description="The command line that the parent process ran")
    ppid = odm.Optional(odm.Integer(), description="The process ID of the parent process")

    pid = odm.Optional(odm.Integer(), description="The process ID")
    image = odm.Text(default="<unknown_image>", description="The image of the process")
    command_line = odm.Optional(odm.Text(), description="The command line that the process ran")
    start_time = odm.Date(description="The time of creation for the process")
    end_time = odm.Date(description="The time of termination for the process")
    integrity_level = odm.Optional(odm.Text(), description="The integrity level of the process")
    image_hash = odm.Optional(odm.Text(), description="The hash of the file run")
    original_file_name = odm.Optional(odm.Text(), description="The original name of the file")
